<?php
// Funktionen zur Zugangskontrolle
// Version 1.0: M. Hauser, Februar 2010
// Version 1.1: M. Hauser, 23.02.2010
//  - Nicht mehr Einlesen der Zugriffsliste in Array und bei jeder Anfrage durchsteppen des Arrays
//    durch den PHP-Interpreter, sondern Aufruf einer (optimierten und in php.exe codierten) String-
//    Suchfunktion (strpos).
// Version 1.2: M. Hauser, 08.04.2010
//  - Geschtzte Kategorien/Seiten mssen ein * am Ende haben
//    Nur fr diesen Fall wird auf Zugriffsrechte geprueft (Resourcen!)
// Version 1.3: M. Hauser 26.04.2010
//  - Beruecksichtigung verschiedener Timeout-Angaben fuer verschiedene Benutzer
// Version 1.3.1: M. Hauser 13.05.2010
//  - Debug-Mglichkeit fr Loginprobleme
// Version 1.3.2: M. Hauser 03.06.2010
//  - Erweiterung des Login-Logs (Passwort durch * ersetzen, Login_OK_Flag)
// Version 1.3.3: M. Hauser 15.09.2010
//  - Gro-/Kleinschreibung vom Benutzernamen ignorieren
// Version 2.0: M. Hauser (mhsob) 05.11.2010
//  - Vollstaendig umprogrammiert, um die Funktionen pluginfaehig zu machen
//    Alle Funktionen komplett ueberarbeitet und Konfigurationsdaten aus plugin.conf eingelesen
//  - Manipulierbarkeit der Cookie-Benutzerinfos erschwert
// --> ab hier Weiterfuehrung der Aenderungshistorie in der Datei doku.html...

if (function_exists('date_default_timezone_set')) date_default_timezone_set('Europe/Berlin');

$ACCESS_CONTROL_CONF = new Properties($PLUGIN_DIR_REL."access_control/plugin.conf");

$AC_ACCESSLIST   = "";
$AC_CURRENT_USER = "";

// ----------------------------------------------
// Hilfsfunktionen
// ----------------------------------------------

// Fuer PHP4...
if (!function_exists('file_put_contents')) {   
  function file_put_contents($filename, $data) {       
    if (($h = @fopen($filename, 'w')) === false) return false;       
    if (($bytes = @fwrite($h, $data)) === false) return false;       
    fclose($h);       
    return $bytes; 
  }
}


function ac_nchr ($char, $count)
{
  $temp = "";
  for ($i=0; $i<$count; $i++) $temp .= $char;
  return $temp;
}

// -----------------------------------------------------------------------------
// Liste mit Zugangsdaten einlesen
//
// Es werden folgende globale Variablen gesetzt;
//
// $AC_ACCESSLIST
// Liste mit allen Kategorien/Seiten, fuer die der aktuell angemeldete Benutzer
// Zugriffsrechte hat
// "<br />Kategorie1'|Seite1'<br />Kategorie2'|Seite2'<br />...<br />"
//
// $AC_CURRENT_USER
// Enthaelt den aktuell angemeldeten Benutzer aus der Session bzw. dem Cookie
// -----------------------------------------------------------------------------
function ac_init_global_variables()
{
  GLOBAL $PLUGIN_DIR_REL;
  GLOBAL $ACCESS_CONTROL_CONF;
  GLOBAL $AC_ACCESSLIST;
  GLOBAL $AC_CURRENT_USER;
  
  $line_separator = "<br />"; // entspr. mehrzeiligem Text in Config-Files

  // Zugangsdaten aus Plugin-Konfiguration laden
  $ac_accesslist_local = explode($line_separator, utf8_decode($ACCESS_CONTROL_CONF->get('ac_accesslist')));
  $ac_users            = explode($line_separator, utf8_decode($ACCESS_CONTROL_CONF->get('ac_users')));

  // Aktuell angemeldeten Benutzer bestimmen (falls anwendbar)
  $AC_CURRENT_USER = $_SESSION['mozilo_user_logged_in'];
  if ($AC_CURRENT_USER == "") $AC_CURRENT_USER = $_COOKIE['mozilo_user_logged_in'];
  $AC_CURRENT_USER_lower = strtolower($AC_CURRENT_USER);
  
  // Benutzerliste fuer spaetere Verwendung zusammenstellen (jeweils Benutzernamen inkl. erste fuenf Stellen des MD5-Passworts)
  // Falls Option gewaehlt, zusaetzlich in Datei exportieren
  $AC_USERLIST = "USERLIST|";
  foreach ($ac_users as $userlist_line)
  {
    if ((substr($userlist_line,0,1)=="#")||($userlist_line == "")) continue; // Kommentar- oder Leerzeile
    $temp = explode('|', $userlist_line);
    $AC_USERLIST .= strtolower($temp[0]).substr($temp[1],0,5) . "|";
  }
  // Pruefen, ob der aktuell angemeldete Benutzer gueltig ist
  if (strpos($AC_USERLIST,"|".$AC_CURRENT_USER_lower."|")===false) 
    $AC_CURRENT_USER = ""; 
  else 
    $AC_CURRENT_USER = substr($AC_CURRENT_USER, 0, strlen($AC_CURRENT_USER)-5); // Fuer weitere Verarbeitung MD5-Passw. abschneiden
  // In Datei mit PHP-Syntax exportieren, falls gewaehlt
  $AC_CURRENT_USER_lower = strtolower($AC_CURRENT_USER);
  if ($ACCESS_CONTROL_CONF->get('export_userlist')== true) file_put_contents($PLUGIN_DIR_REL."access_control/ac_valid_users.php", "<"."?php \$valid_users = \"".$AC_USERLIST."\"; ?".">");
  
  // Zugriffsliste zusammenstellen
  // falls kein Benutzer angemeldet ist, alles ausser Kommentarzeilen aus der Plugin-Konfiguration uebernehmen
  // falls ein Benutzer angemeldet ist, nur diejenigen Zeilen uebernehmen, welche fuer ihn relevant sind
  $AC_ACCESSLIST = $line_separator;
  // Fuer die weitere Verarbeitung MD5-Passwort hinten abschneiden und durch Komma vorne und hinten Suchergebnis eindeutig machen
  foreach ($ac_accesslist_local as $accesslist_line)
  {
    if ((substr($accesslist_line,0,1)=="#")||($accesslist_line == "")) continue; // Kommentar- oder Leerzeile
    if ($AC_CURRENT_USER != "")
    {
      $temp = explode('|',$accesslist_line);
      $temp[2] = ",".strtolower ($temp[2]).","; // Liste der Benutzer, die Zugriff auf Kategorie|Seite haben (,user1,user2,user3,)
      if ( (strpos($temp[2],",any_login,") === false) && (strpos($temp[2], ",".$AC_CURRENT_USER_lower.",") === false) ) continue; // Zeile betrifft aktuell angemeldeten Benutzer nicht
      $accesslist_line = $temp[0]."|".$temp[1];
    }
    $AC_ACCESSLIST .= $accesslist_line . $line_separator;
  }
  //file_put_contents ("debug.txt", "AC_ACCESSLIST=".print_r ($AC_ACCESSLIST, true)."\n\nAC_CURRENT_USER=".$AC_CURRENT_USER."\nAC_USER_VALID=".$AC_USER_VALID."\ncurrent_username=". $current_username."\n\n".print_r ($AC_USERLIST,true));
}


// -----------------------------------------------------------------------------
// Prfen, ob der aktuell angemeldete Benutzer Zugang zur uebergebenen
// Kategorie / Seite hat.
// Die Funktion wird nur aufgerufen, wenn das letzte Zeichen des Kategorie- bzw.
// Seitennamens ein ' ist.
//
// Returnwert 0=kein Zugang, 1=Zugang
// -----------------------------------------------------------------------------
function ac_verify_access($category,$content)
{
  GLOBAL $AC_ACCESSLIST;
  GLOBAL $AC_CURRENT_USER;
  
  if ($AC_CURRENT_USER == "") return 0;
  if ($content == "")
  {
    // nach ">KATEGORIE|" suchen
    if (strpos($AC_ACCESSLIST, ">".html_entity_decode($category)."|") !== false) return 1;
  }
  else
  {
    // nach ">KATEGORIE|SEITE<" suchen (> und < kommen von <br />
    if (strpos($AC_ACCESSLIST, ">".html_entity_decode($category)."|".html_entity_decode($content)."<") !== false) return 1;
  }  
  return 0;
}


// -----------------------------------------------------------------------------
// berprfen, ob die eingegebenen Zugangsdaten korrekt sind
//
// Returnwert !=0 korrekt, 0=falsch
// -----------------------------------------------------------------------------
function ac_logincheck($user, $password)
{
  GLOBAL $ACCESS_CONTROL_CONF;
  GLOBAL $PLUGIN_DIR_REL;

  $login_ok = 0;
  $ac_users = strtolower(utf8_decode($ACCESS_CONTROL_CONF->get('ac_users')));
  $user     = utf8_decode($user);
  $password = utf8_decode($password);
  // In der Benutzerliste nach "USER|MD5PASSWORD|..." mit ... in {-,0..9} suchen und ... zurueckgeben
  //preg_match ('/<br \/>'.strtolower($user).'\|'.md5($password).'\|([\-\d]+)/', $ac_users, $matches);
  preg_match ('/<br \/>'.preg_quote(strtolower($user),'/').'\|'.md5($password).'\+?\|([\-\d]+)/', $ac_users, $matches);
  if ($matches[1]!="") $login_ok = $matches[1];
  //file_put_contents('debug2.txt', strtolower($user)." ".md5($password)."\n\n".$ac_users);

  // Logging
  if ($ACCESS_CONTROL_CONF->get('login_log_enable')=='true')
  {
    // Pruefen, ob der aktuelle Benutzer evtl. vom Logging ausgeschlossen wurde
    if (array_search(  strtolower($user), explode( ',', strtolower(utf8_decode($ACCESS_CONTROL_CONF->get('login_log_nolog_userlist'))) )  ) === FALSE)
    {
      if ($login_ok) $login_ok_flag = "Login OK"; else $login_ok_flag = "Login FEHLER"; 

      // Speichern in Protokoll-Datei
      $file = $PLUGIN_DIR_REL."access_control/ac_log.txt";
      if (is_file($file)) $all_lines = file($file, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES); else $all_lines = array();
      for ($i=count($all_lines); $i>0; $i--) {$all_lines[$i] = rtrim ($all_lines[$i-1]);} // u.a. da FILE_IGNORE_NEW_LINES nicht zuverlaessig je nach PHP-Version 
      $all_lines[0] = date('d.m.Y  H:i:s')."|".$user."|".ac_nchr("*", strlen($password))."|".$login_ok_flag;
      $login_log_length = $ACCESS_CONTROL_CONF->get('login_log_length'); if ($login_log_length == "") $login_log_length = 100;
      if (count($all_lines)>$login_log_length) $all_lines = array_slice ($all_lines, 0, $login_log_length);
      file_put_contents ($PLUGIN_DIR_REL."access_control/ac_log.txt", implode("\r\n",$all_lines));
      
      // optional: Email senden
      if ($ACCESS_CONTROL_CONF->get('login_log_email_enable')=='true')
      {
        $empfaenger = str_replace("<br />", ",", $ACCESS_CONTROL_CONF->get('login_log_email_receivers'));
        $betreff = $ACCESS_CONTROL_CONF->get('login_log_email_subject');
        $nachricht = date('d.m.Y  H:i:s')."\n\nBenutzer: ".$user."\nStatus:   ".$login_ok_flag."\n";
        $header = 'From: '. $ACCESS_CONTROL_CONF->get('login_log_email_sender') . "\r\n" .
                  'X-Mailer: PHP/' . phpversion();
        //file_put_contents ("debug.txt", $empfaenger."\n".$betreff."\n".$nachricht."\n".$header);
        mail($empfaenger, $betreff, $nachricht, $header);
      }
    }
  }
  return $login_ok;
}


?>